import { ethers } from "ethers";

// rpc 설정 골리
const provider = new ethers.JsonRpcProvider("https://goerli.infura.io/v3/50e195e9e1cb48dba3b50c212198bc7e");

// rpc 설정 베이스
//const provider = new ethers.JsonRpcProvider("https://base.llamarpc.com");

// 지갑 설정
const signer = new ethers.Wallet("1번지갑", provider);

//atomic_base abi
const contractABI = [
    {
        "type": "function",
        "name": "propose",
        "inputs": [
          {
            "name": "proposal",
            "type": "tuple",
            "internalType": "struct PartyGovernance.Proposal",
            "components": [
              {
                "name": "maxExecutableTime",
                "type": "uint40",
                "internalType": "uint40"
              },
              {
                "name": "cancelDelay",
                "type": "uint40",
                "internalType": "uint40"
              },
              {
                "name": "proposalData",
                "type": "bytes",
                "internalType": "bytes"
              }
            ]
          },
          {
            "name": "latestSnapIndex",
            "type": "uint256",
            "internalType": "uint256"
          }
        ],
        "outputs": [
          {
            "name": "proposalId",
            "type": "uint256",
            "internalType": "uint256"
          }
        ],
        "stateMutability": "nonpayable"
    }
];

//골리
const contractAddress = '0xa8f0df2fa7b5f38386d86e0e0bcab6b29e2771eb';
//베이스
//const contractAddress = '0x755793ca421e054d4c62797319e5409414bd21a4';

// 컨트랙트 객체 생성
const contract = new ethers.Contract(contractAddress, contractABI, signer);

async function encodeDistributeProposalData(amount, tokenType, token, tokenId) {
    const encoder = ethers.AbiCoder.defaultAbiCoder();
    // DistributeProposalData 구조체의 각 필드를 인코딩
    return encoder.encode(
        ['uint256', 'uint8', 'address', 'uint256'],
        [amount, tokenType, token, tokenId]
    );
}


async function propose() {
    const nowMillis = Date.now();
    const nowSeconds = Math.floor(nowMillis / 1000);

    // ProposalData 예시 생성
    // const proposerType = 7; // ProposalType 7: DistributeProposal <- 잘못됨
    const amount = 100000000000000; // 0.0001이더
    const tokenType = 0; // Erc20 타입
    const token = '0xeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeee'; // 예시: 토큰 주소
    const tokenId = 0; // 예시: 토큰 ID  

    const proposalData = "0x00000007" + (await encodeDistributeProposalData(amount, tokenType, token, tokenId)).substring(2); //"0x00000007": DistributeProposal는 7번 프로포절

    // Proposal 데이터 인코딩
    const proposal = {
        maxExecutableTime: 1709294807, // maxExecutableTime nowSeconds + 3628800가 기본설정 근데 이걸 execute에서 참조해야돼서 그냥 상수로 둘게요
        cancelDelay: 3628800,       // cancelDelay 기본설정
        proposalData: proposalData
    };

    // 최신 스냅샷 인덱스 설정
    const latestSnapIndex = 0;
    
    const callData = contract.interface.encodeFunctionData("propose", [
        proposal,
        latestSnapIndex
    ]);
    console.log(callData);
    //propose 함수 호출
    try {
        // 가스비 설정
        const gasLimit = 700000; // 예: 100,000 가스 한도
        const gasPrice = 100000000000; // 예: 50 gwei

        const tx = await contract.propose(proposal, latestSnapIndex, {
            gasLimit: gasLimit,
            gasPrice: gasPrice
        });

        console.log('Transaction sent! Hash:', tx.hash);
        await tx.wait();
        console.log('Transaction confirmed.');
    } catch (error) {
        console.error('Transaction failed:', error);
    }
}

await propose();